#!/bin/sh
# $Id: MRTGgen.tcl,v 1.3 2001/05/11 10:37:01 gdmr Exp $

# Copyright (c) 2000 Division of Informatics, University of Edinburgh
# 
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of the University of Edinburgh not be used
# in advertising or publicity pertaining to distribution of the software
# without specific, written prior permission.  The University of Edinburgh
# makes no representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied warranty.
#
# Comments are invited.  Please send to gdmr@dcs.ed.ac.uk.

# Generate MRTG configuration files from the switch configuration.  Write
# it out to stdout, so our caller can decide where to put it.

# tclsh is in a different place on Solaris and Linux, and we can't
# assume that it's in our PATH as we may be running from a cron job.
# This is the standard tcl hack, only with more than one shell line...
#\
case "`uname -sr`" in
#\
	'SunOS 5'*)	TCLSH=/usr/local/bin/tclsh ;;
#\
	Linux*)		TCLSH=/usr/tcl8/bin/tclsh ;;
#\
	 *)		echo "Don't know where to find tclsh" ; exit ;;
#\
esac
#\
exec $TCLSH "$0" "$@"
# It's tcl from here on...

# Auto-load from the directory that this script lives in
lappend auto_path [file dirname $argv0]

# Global configuration settings
set workDir "/disk/home/farg/gdmr/mrtg/mrtgWork"

# The next few variables set the values for the "extended host name syntax"
# (with "" meaning to use the defaults).
set ePort ""
set eTimeout ""
set eRetries "2"
set eBackoff ""
set eVersion ""

# RCS id for later use
set rcsId {$Id: MRTGgen.tcl,v 1.3 2001/05/11 10:37:01 gdmr Exp $}

#
# Get the switch configuration name from the command line
# and any flags
#
proc Usage {} {
   global argv0
   puts stderr "$argv0 config-file"
}

set debug 0
foreach a $argv {
   switch -glob -- $a {
      -debug    { set debug 1 }
      -*	{ Usage ; exit 1 }
      *		{ if [info exists configFile] { Usage ; exit 1 }
		  set configFile $a }
   }
}

if {![info exists configFile]} {
   Usage
   exit 1
}

# What's legal in the configuration file?
set confCommands [parseCommands]

# Read the configuration file
proc readConfig {configFile confCommands} {
   global debug
   if {$debug} {
      puts stderr "Reading configuration file \"$configFile\"..."
   }
   if [catch {open $configFile r} cf] {
      puts stderr "Read switch configuration: $cf"
      exit 2
   } else {
      if [catch {split [read $cf] "\n"} confFile] {
         puts stderr "Error reading $configFile: $confFile"
         exit 2
      }
      close $cf
   }
   # Parse it if it read in OK
   set bogusCommand 0
   foreach confLine $confFile {
      set confLine [string trim $confLine]
      if [string length $confLine] {
         set cfw [lindex [split $confLine] 0]
         if [string compare [string index $cfw 0] "#"] {
            # Not a comment, so...
            if {![string compare $cfw "portfile"]} {
               # Read a sub-file.
               # ("eval" loses any quotes there might be)
               set pfn [lindex [split $confLine] 1]
               eval readConfig $pfn [list "port"]
            } elseif {![string compare $cfw "vlanfile"]} {
               # Read a sub-file.
               # ("eval" loses any quotes there might be)
               set pfn [lindex [split $confLine] 1]
               eval readConfig $pfn [list "vlan"]
            } elseif {[lsearch -exact $confCommands $cfw] >= 0} {
               # Validated against legal commands
               if [catch {eval $confLine} problem] {
                  puts stderr "Error in \"$confLine\""
                  if {$debug} {puts stderr "$problem"}
                  set bogusCommand 1
               }
            } else {
               # Something we don't recognise
               puts stderr "Unknown/disallowed configuration option \"$cfw\""
               set bogusCommand 1
            }
         }
      }
   }
   # Did it parse?
   if {$bogusCommand || ![parseOK]} {
      puts stderr "Terminating due to errors in $configFile"
      exit 2
   }
}

readConfig $configFile $confCommands
set cC [file split $configFile]
set configName [file rootname [lindex $cC [expr {[llength $cC] - 1}]]]
set target [getTarget]
set community [getCommunity]

# Initialise SNMP and grab the interface names and indices
snmpInit $target $community
snmpEtherIfIndices

# Also the switch name and location
set which [snmpName]
set where [snmpLocation]
#puts stderr "Switch $which located $where"

# Generate configuration header and any global options
puts "# Switch: $which, location: $where"
puts "# Generated from $configFile on [clock format [clock seconds]]"
puts "# by $rcsId"
puts ""
puts "WorkDir: $workDir"
puts "Options\[_\]: growright, unknaszero"
puts ""

# Find a named interface's speed.  Pretend it's a Gbit link if
# we can't find out.
proc portSpeed {p} {
   if [catch {snmpIfSpeed [snmpIfIndex $p]} s] {
      puts stderr "$p: snmpIfSpeed failed: $s"
      puts stderr "Pretending $p is a Gbit link"
      return 125000000
   } else {
      set s
   }
}

# Process one entry: generate the appropriate MRTG configuration
proc processOne {port name} {
   global debug
   global configName
   global target
   global community
   global ePort
   global eTimeout
   global eRetries
   global eBackoff
   global eVersion
   if {$debug} {
      puts stderr "$configName: $port -> $name"
   }
   set tag "${configName}_$port"
   set eStuff ":$ePort:$eTimeout:$eRetries:$eBackoff:$eVersion"
   set speed [portSpeed $port]
   puts "#-#-# $name -> $tag"
   puts "Target\[$tag\]: #$port:$community@$target$eStuff"
   puts "MaxBytes\[$tag\]: [expr {$speed / 8}]"
   puts "Title\[$tag\]: $name ($port on $configName)"
   puts "PageTop\[$tag\]: <H1>Traffic analysis for $name (port $port on switch $configName)</H1>"
   puts " $name is running at [expr {$speed / 1000000}]Mbps"
   puts "LegendI\[$tag\]: &nbsp;From:"
   puts "LegendO\[$tag\]: &nbsp;To:"
   puts ""
}

# Process everything from the configuration file
foreach c [getPortList] {
   processOne $c [getPortName $c]
}

exit 0
