# $Id: parse.tcl,v 3.0 2001/05/10 09:57:15 gdmr Exp $
#
# Copyright 2000 Division of Informatics, University of Edinburgh
# 
# Permission to use, copy, modify, distribute, and sell this software and its
# documentation for any purpose is hereby granted without fee, provided that
# the above copyright notice appear in all copies and that both that
# copyright notice and this permission notice appear in supporting
# documentation, and that the name of the University of Edinburgh not be used
# in advertising or publicity pertaining to distribution of the software
# without specific, written prior permission.  The University of Edinburgh
# makes no representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied warranty.
#
# Procedures used to parse the switch definition files.  These are structured
# in such a way that they're basically a collection of Tcl calls to things
# defined in this file.  There isn't a separate parser for them...

#
# Things it's sensible to have in a config-file
# Make this a proc so that calling it causes this
# file to be auto-loaded
#
proc parseCommands {} {
   list target community port portskip portlock vlan spanningtree
}

#
# parse error reporting
#
set parseError 0

proc parseOK {} {
   global parseError
   if {$parseError} {
      return 0
   } else {
      return 1
   }
}

#
# General switch definitions
#
proc target {t} {
   global parseTarget
   if [info exists parseTarget] {
      puts stderr "Redefining target switch: $parseTarget -> $t"
   }
   set parseTarget $t
}

proc getTarget {} {
   global parseTarget
   set parseTarget
}

proc community {c} {
   global parseCommunity
   if [info exists parseCommunity] {
      puts stderr "Redefining community: $parseCommunity -> $c"
   }
   set parseCommunity $c
}

proc getCommunity {} {
   global parseCommunity
   set parseCommunity
}

#
# port definitions and things
#
set parseSkipList {}

proc portskip {p} {
   global parseSkipList
   lappend parseSkipList [string toupper $p]
}

proc getSkipList {} {
   global parseSkipList
   set parseSkipList
}

set parseLockList {}

proc portlock {p} {
   global parseLockList
   lappend parseLockList [string toupper $p]
}

proc getLockList {} {
   global parseLockList
   set parseLockList
}

set parseLegalSpeeds {KEEP - 10HD 10FD 100HD 100FD}

proc port {p name {speed -} {vlan default} args} {
   global parseName
   global parseSpeed
   global parseLegalSpeeds
   global parseDefVlan
   global parseVlans
   global parseVlanTags
   global parseVlanTagList
   global parseError
   set p [string toupper $p]
   if [info exists parseName($p)] {
      puts stderr "Redefining port $p: $name $speed $vlan $args"
      set parseError 1
      return
   }
   set parseName($p) $name
   set us [string toupper $speed]
   if {[lsearch -exact $parseLegalSpeeds $us] >= 0} {
      set parseSpeed($p) $us
   } else {
      puts stderr \
         "Speed \"$speed\" for port $p not in \"$parseLegalSpeeds\""
      set parseError 1
   }
   if [info exists parseVlanTags($vlan)] {
      set parseDefVlan($p) $parseVlanTags($vlan)
   } else {
      puts stderr "Undefined VLAN \"$vlan\" for port $p"
      set parseError 1
   }
   set utv {}
   if {[llength $args] == 1 && ![string compare [lindex $args 0] "*"]} {
      foreach v [lsort -integer $parseVlanTagList] {
         if {$v != $parseDefVlan($p)} {
            lappend utv $v
         }
      }
   } else {
      foreach v $args {
         if {[info exists parseVlanTags($v)]} {
            lappend utv $parseVlanTags($v)
         } else {
            puts stderr "Undefined VLAN \"$v\" for port $p"
            set parseError 1
         }
      }
   }
   set parseVlans($p) $utv
}

proc getPortList {} {
   global parseName
   return [lsort [array names parseName]]
}

proc portDefined {p} {
   global parseName
   set p [string toupper $p]
   info exists parseName($p)
}

proc getPortName {p} {
   global parseName
   set p [string toupper $p]
   if [info exists parseName($p)] {
      set parseName($p)
   } else {
      return ""
   }
}

proc getPortSpeed {p} {
   global parseSpeed
   set p [string toupper $p]
   # Beware: if the port hasn't been configured then
   # we won't have a speed for it.  In this case the
   # easiest thing is to assume that it's auto.
   if [info exists parseSpeed($p)] {
      set parseSpeed($p)
   } else {
      return "-"
   }
}

#
# VLAN definitions
#
set parseVlanTags(default) 1
set parseVlanTags("-") 1

proc getPortDefVlan {p} {
   global parseDefVlan
   set p [string toupper $p]
   if [info exists parseDefVlan($p)] {
      set parseDefVlan($p)
   } else {
      return "1"
   }
}

proc getPortVlans {p} {
   global parseVlans
   set p [string toupper $p]
   if [info exists parseVlans($p)] {
      lsort -integer $parseVlans($p)
   } else {
      return ""
   }
}

set parseVlanTagList {1}
set parseVlanNameList {"default"}

proc vlan {name tag {fullName ""}} {
   global parseVlanTags
   global parseVlanFullNames
   global parseVlanTagList
   global parseVlanNameList
   global parseError
   if {[lsearch -exact $parseVlanTagList $tag] >= 0} {
      puts stderr "Duplicate VLAN tag: $tag"
      set parseError 1
      return
   }
   if {[lsearch -exact $parseVlanNameList $name] >= 0} {
      puts stderr "Duplicate VLAN name: $name"
      set parseError 1
      return
   }
   lappend parseVlanTagList $tag
   lappend parseVlanNameList $name
   set parseVlanTags($name) $tag
   if {[string length $fullName] == 0} {
      set fullName $name
   }
   regsub -all " " $fullName "-" fullName
   set parseVlanFullNames($tag) $fullName
}

proc getVlanFullName {t} {
   global parseVlanFullNames
   if [info exists parseVlanFullNames($t)] {
      set parseVlanFullNames($t)
   } else {
      return "*unknown vlan name*"
   }
}

proc getVlanTagList {} {
   global parseVlanTagList
   lsort -integer $parseVlanTagList
}

#
# Spanning Tree Protocol definitions
#
set parseStpMode("default") "normal"
set parseStpValidModes {"normal" "fast"}
set parseStpCost("default") -1

set parseSptWanted 0
proc spanningtree {p mode {cost -1}} {
   global parseError
   global parseStpMode
   global parseStpValidModes
   global parseStpCost
   global parseStpWanted
   global parseName
   set mode [string tolower $mode]
   if {[lsearch -exact $parseStpValidModes $mode] >= 0} {
      if {[string compare "$p" "default"]} {
         set p [string toupper $p]
         if [info exists parseName($p)] {
            set parseStpMode($p) $mode
            set parseStpCost($p) $cost
         } else {
            puts stderr "spanningtree: no port $p defined"
            set parseError 1
         }
      } else {
         set parseStpMode("default") $mode
         set parseStpCost("default") $cost
      }
      set parseStpWanted 1
   } else {
      puts stderr "spanningtree: unknown mode $mode for $p"
      set parseError 1
   }
}

proc getStpWanted {} {
   global parseStpWanted
   set parseStpWanted
}

proc getStpMode {p} {
   global parseStpMode
   set p [string toupper $p]
   if [info exists parseStpMode($p)] {
      set parseStpMode($p)
   } else {
      set parseStpMode("default")
   }
}

proc getStpCost {p} {
   global parseStpCost
   set p [string toupper $p]
   if [info exists parseStpCost($p)] {
      set parseStpCost($p)
   } else {
      set parseStpCost("default")
   }
}
